//
//  MyAppController.m
//  PboardPeeker
//
//  Copyright (c) 2003 MIKOZAWA Akira. All rights reserved.
//

#import "MyAppController.h"

#define		key_FontName	@"PBP-FontName"
#define		key_FontSize	@"PBP-FontSize"
#define		key_ShowData	@"PBP-ShowData"
#define		key_PboardName	@"PBP-PboardName"
#define		Key_Window		@"PBP-Window"

@interface MyAppController ( Private )
- (NSString*)pboardNameForCode:(NSString*)aCode;
- (NSString*)pboardCode;
- (void)setPboardCode:(NSString*)aCode;
- (void)notifiedFromDumper:(NSNotification*)notification;
- (void)checkUpdateDone:(id)dummy;
- (void)setupButtons:(BOOL)flag;
@end // @interface MyAppController ( Private )

@implementation MyAppController

-(void)dealloc
{
	[mDumper release];
	[mPboardCode release];
	NSNotificationCenter*	nc = [NSNotificationCenter defaultCenter];
	[nc removeObserver:self];
	[super dealloc];
}

- (IBAction)update:(id)sender
{
	if ( mUpdating ) {
		[mDumper interrupt];
	}
	else {
		NSRect	frame = [myTextView frame];
		[myTextView setNeedsDisplayInRect:frame];
		NSString*	msg = NSLocalizedString(@"Updating...", @"");
		[myTextView setString:msg];
	
		NSString*	pbName = [self pboardNameForCode:[self pboardCode]];
		NSPasteboard*	pboard = [NSPasteboard pasteboardWithName:pbName];
	
		[self setupButtons:NO];

		mUpdating = YES;
		mTimer = [NSTimer 
					scheduledTimerWithTimeInterval:0.1 
					target:self 
					selector:@selector(checkUpdateDone:) 
					userInfo:nil 
					repeats:YES];
		
		mDumper = [[MyPBoardDumper alloc] initWithDataDump:mShowData];
		[NSThread detachNewThreadSelector:@selector(startForPboard:) 
				toTarget:mDumper withObject:pboard];
	}
}

- (IBAction)selectPboard:(id)sender
{
	[self setPboardCode:[sender titleOfSelectedItem]];
	[self update:self];
}

- (IBAction)toggleDataDisplay:(id)sender
{
	mShowData = [sender state] == NSOnState;
	[self update:self];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
	NSUserDefaults*	usrDfts = [NSUserDefaults standardUserDefaults];

	NSString*	fontName = @"Monaco";
	NSString*	fontSize = @"10.0";
	NSString*	dispData = @"0";
	[self setPboardCode:@"NSGeneralPboard"];
	
	NSString*	usrDftVal = [usrDfts stringForKey:key_FontName ];
	if ( usrDftVal != nil )
		fontName = usrDftVal;
	usrDftVal = [usrDfts stringForKey:key_FontSize];
	if ( usrDftVal != nil )
		fontSize = usrDftVal;
	NSFont*	theFont = [NSFont fontWithName:fontName size:[fontSize doubleValue]];
	[myTextView setFont:theFont];
	
	usrDftVal = [usrDfts stringForKey:key_ShowData ];
	if ( usrDftVal != nil )
		dispData = usrDftVal;
	mShowData = [dispData isEqual:@"1"];
	[myShowDataChkbox setState:mShowData];
	
	usrDftVal = [usrDfts stringForKey:key_PboardName ];
	if ( usrDftVal != nil ) {
		[self setPboardCode:usrDftVal];
	}
	[myPbNamePopup selectItemWithTitle:[self pboardCode]];

	NSWindow*		theWindow = [myTextView window];
	[theWindow setFrameAutosaveName:Key_Window];
	
	mUpdating = NO;
	NSNotificationCenter*	nc = [NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(notifiedFromDumper:) 
						name:myNotification_DumpEnd object:nil];

	[self update:self];
}

- (void)applicationWillTerminate:(NSNotification *)notification
{
	NSUserDefaults*	usrDfts = [NSUserDefaults standardUserDefaults];
	[usrDfts setBool:mShowData forKey:key_ShowData ];
	[usrDfts setObject:[self pboardCode] forKey:key_PboardName ];

	NSFont*		theFont = [myTextView font];
	if ( theFont ) {
		NSString*	fontName = [theFont fontName];
		CGFloat		fontSize = [theFont pointSize];
		[usrDfts setObject:fontName forKey:key_FontName ];
		[usrDfts setFloat:fontSize forKey:key_FontSize ];
	}

	[usrDfts synchronize];
}

- (void)applicationWillBecomeActive:(NSNotification *)notification
{
	NSWindow*		theWindow = [myTextView window];
	[theWindow makeKeyAndOrderFront:self];
}

- (BOOL)windowShouldClose:(id)sender
{
	NSApplication*	app = [NSApplication sharedApplication];
	[app terminate:self];
	return YES;
}

@end // @implementation MyAppController

@implementation MyAppController ( Private )

- (NSString*)pboardNameForCode:(NSString*)aCode;
{
	NSString*	name = nil;
	if ( [aCode isEqual:@"NSFontPboard"] )
		name = NSFontPboard;
	else if ( [aCode isEqual:@"NSRulerPboard"] )
		name = NSRulerPboard;
	else if ( [aCode isEqual:@"NSFindPboard"] )
		name = NSFindPboard;
	else if ( [aCode isEqual:@"NSDragPboard"] )
		name = NSDragPboard;
	else
		name = NSGeneralPboard;
	return name;
}

- (NSString*)pboardCode
{
	return mPboardCode;
}

- (void)setPboardCode:(NSString*)aCode
{
	[mPboardCode release];
	mPboardCode = nil;
	
	if ( !( [aCode isEqual:@"NSGeneralPboard"] ||
			[aCode isEqual:@"NSFontPboard"] ||
			[aCode isEqual:@"NSRulerPboard"] ||
			[aCode isEqual:@"NSFindPboard"] ||
			[aCode isEqual:@"NSDragPboard"] ) ) 
	{
		aCode = @"NSDragPboard";
	}
	mPboardCode = [[NSString alloc] initWithString:aCode];
}

- (void)notifiedFromDumper:(NSNotification*)notification
{
	mUpdating = NO;
}

- (void)checkUpdateDone:(id)dummy
{
	if ( ! mUpdating ) {
		[mTimer invalidate];
		NSString*	str = [mDumper string];
		[myTextView setString:str];
		[mDumper release];
		mDumper = nil;
		[self setupButtons:YES];
	}
}

- (void)setupButtons:(BOOL)flag
{
	NSString*	title = flag ? @"Update" : @"Cancel";
	NSString*	btnTitle = NSLocalizedString(title, @"");
	[myUpdateButton setTitle:btnTitle];
	[myShowDataChkbox setEnabled:flag];
	[myPbNamePopup setEnabled:flag];
}

@end // @implementation MyAppController ( Private )
