//
//  MyPBoardDumper.m
//  PboadPeeker
//
//  Copyright (c) 2003 MIKOZAWA Akira. All rights reserved.
//

#import "MyPBoardDumper.h"

@interface MyPBoardDumper ( Private )
- (NSString*)stringForType:(NSString*)aType data:(NSData*)aData;
- (NSString*)stringForData:(NSData*)aData;
@end // @interface MyPBoardDumper ( Private )


@implementation MyPBoardDumper

-(id) initWithDataDump:(BOOL)flag
{
	[super init];
	string = [[NSMutableString alloc] initWithCapacity:0];
	allowProcessing = YES;
	showData = flag;
	return self;
}

-(void) dealloc
{
	[string release];
	[super dealloc];
}

-(void) startForPboard:(NSPasteboard*)pboard
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];

	NSString*		pbName = [pboard name];
	[string appendFormat:@"Name: %@\r", pbName];

	NSArray*		pbTypes = [pboard types];
	NSUInteger		typeCount = [pbTypes count];
	
	[string appendFormat:@"number of types: %ld\r\r", (long)typeCount];

	for ( NSUInteger i = 0; i < typeCount; i++ ) {
		NSAutoreleasePool*	pool2 = [[NSAutoreleasePool alloc] init];
		NSString*	theType = [pbTypes objectAtIndex:i];
		NSData*		theData = [pboard dataForType:theType];
		[string appendString:[self stringForType:theType data:theData]];

		if ( showData ) {
			[string appendString:[self stringForData:theData]];
			[string appendString:@"\r"];
		}
		[pool2 release];
	}

	NSNotificationCenter*	nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:myNotification_DumpEnd object:self];
	
	[pool release];
}

-(void) interrupt
{
	allowProcessing = NO;
}

-(NSString*) string
{
	return string;
}
@end // @implementation MyPBoardDumper


@implementation MyPBoardDumper ( Private )

static NSString* OSTypeString ( NSString* aString );

- (NSString*)stringForType:(NSString*)aType data:(NSData*)aData
{
	NSMutableString*	theText = [NSMutableString stringWithCapacity:0];

	[theText appendFormat:@"type: '%@'", aType];

	if ( [aType hasPrefix:@"CorePasteboardFlavorType 0x"] ) {
		NSUInteger	len0 = [aType length];
		NSUInteger	len1 = [@"CorePasteboardFlavorType 0x" length];
		if ( len0 == len1+8 ) {
			NSRange	range = NSMakeRange ( len1, 8 );
			NSString*	osTypeHexStr = [aType substringWithRange:range];
			NSString* 	osTypeStr = OSTypeString ( osTypeHexStr );
			[theText appendFormat:@" ('%@')", osTypeStr];
		}
	}
	
	[theText appendFormat:@"  size: %ld\r", (long)[aData length] ];
	
	return theText;
}

- (NSString*)stringForData:(NSData*)aData
{
	NSUInteger	len = [aData length];
	const unsigned char*	bytes = [aData bytes];
	NSMutableString*	theText = [NSMutableString stringWithCapacity:0];
	
	for (NSUInteger i = 0; allowProcessing && i < len; ) {
		NSUInteger	start = i;
		NSUInteger	len2 = 16;
		if ( start + len2 > len )
			len2 = len - start;
		[theText appendFormat:@"%08lX: ", (long)start];
		for ( NSUInteger j = 0; j < len2; j++ ) {
			NSInteger		dd = bytes [ start + j ];
			[theText appendFormat:@"%02lX", (long)dd];
			if ( j % 2 == 1 )
				[theText appendString:@" "];
		}
		if ( len2 < 16 ) {
			for ( NSUInteger j = len2; j < 16; j++ ) {
				[theText appendString:@"  "];
				if ( j % 2 == 1 )
					[theText appendString:@" "];
			}
		}
		[theText appendString:@" '"];
		for ( NSUInteger j = 0; j < len2; j++ ) {
			NSInteger		dd = bytes [ start + j ];
			if ( isprint ( dd ) )
				[theText appendFormat:@"%1c", dd];
			else 
				[theText appendString:@"."];
		}
		[theText appendString:@"'\r"];
		i += len2;
	}
	return theText;
}

@end // @implementation MyPBoardDumper ( Private )

//==================================================================
// 16進 1桁を数値に変換
// "4" >> 4

static NSInteger toDigit ( unichar aChar )
{
	NSInteger	digit = 0;
	NSInteger ch = toupper ( aChar );

	if ( '0' <= ch && ch <= '9' ) {
		digit = ch - '0';
	}
	else if ( 'A' <= ch && ch <= 'F' ) {
		digit = ch - 'A' + 10;
	}

	return digit;
}

// 16進 2桁を文字に変換
// "41" >> "A"

static NSString* toChar ( NSString* aString )
{
	NSString*	newStr = nil;
	NSUInteger	len = [aString length];
	if ( len == 2 ) {
		unichar	v = toDigit ( [aString characterAtIndex:0] ) * 16 +
					toDigit ( [aString characterAtIndex:1] );
		newStr = [NSString stringWithCharacters:&v length:1];
	}
	return newStr;
}

// 16進表記の 4 char code を文字表記に変換
// "41424344" >> "ABCD"

static NSString* OSTypeString ( NSString* aString )
{
	NSString*	newStr = nil;
	NSUInteger	len = [aString length];
	if ( len == 8 ) {
		NSMutableString*	temp = [NSMutableString stringWithCapacity:4];
		for ( NSInteger i = 0; i < 4; i++ ) {
			NSRange	range = NSMakeRange ( i*2, 2 );
			NSString*	digStr = [aString substringWithRange:range];
			NSString*	chStr = toChar ( digStr );
			[temp appendString:chStr];
		}
		newStr = temp;
	}
	return newStr;
}
